cmake_minimum_required (VERSION 3.18)

# Use new policy to use CMAKE_CXX_STANDARD in trysight_compile() macro
cmake_policy(SET CMP0067 NEW)

set(FW_BUILD_EXTERNAL 1)

include(GNUInstallDirs)

include("${CMAKE_CURRENT_LIST_DIR}/build/flags.cmake")
include("${CMAKE_CURRENT_LIST_DIR}/macros.cmake")

if(NOT SIGHT_CONFIGURED)
    set(SIGHT_CONFIGURED TRUE)
    # Needed for FindFilesystem.cmake
    list(APPEND CMAKE_MODULE_PATH ${CMAKE_CURRENT_LIST_DIR})

    set(SIGHT_SUPPORTED_COMPONENTS core;data;activity;service;atoms;utest;utestData;geometry_eigen;geometry_data;geometry_glm;geometry_vision;io_base;io_itk;filter_vision;filter_image;filter_dicom;ui_history;ui_base;ui_qml;ui_qt;io_zip;io_http;io_atoms;io_patch;io_vtk;io_opencv;io_dimse;viz_base;viz_qt3d;viz_scene2d;viz_scene3d;module_memory;module_sync;module_debug;module_service;module_data;module_utest;module_filter_image;module_filter_point;module_filter_vision;module_filter_mesh;module_geometry_vision;module_geometry_base;module_io_video;module_io_itk;module_io_vision;module_io_vtk;module_io_atoms;module_io_dicomweb;module_io_document;module_io_matrix;module_navigation_calibration;module_navigation_optics;module_ui_debug;module_ui_console;module_ui_base;module_ui_flaticons;module_ui_qml;module_ui_icons;module_ui_qt;module_ui_history;module_viz_scene2d;module_viz_scene3d;module_viz_scene3dQt;module_viz_scene3d_test;module_viz_sample;module_viz_qt3d;config_viz_scene2d;config_ui_datamanager;geometry_vtk;ui_dicom;io_dicom;io_igtl;io_session;module_appXml;module_activity;module_io_dimse;module_io_igtl;module_io_session;module_ui_viz;module_ui_dicom;config_viz_scene3d;activity_navigation_calibration;module_io_activity;activity_viz_negato;activity_viz_3d;module_io_dicom;config_viz_image;config_ui_activity;activity_io_dicom;activity_io_dicomweb)

    if(UNIX)
        configure_file(
            ${FWCMAKE_RESOURCE_PATH}/install/linux/safe-xvfb-run 
            ${CMAKE_BINARY_DIR}/bin/safe-xvfb-run
            COPYONLY
        )
        list(APPEND CMAKE_PREFIX_PATH /usr/share/OGRE/cmake/modules)
        list(APPEND CMAKE_PREFIX_PATH ${FWCMAKE_RESOURCE_PATH}/modules)
    endif()

    set(FW_INSTALL_PATH_SUFFIX "${PROJECT_NAME}")
    set(FWCONFIG_PACKAGE_LOCATION lib/cmake/${PROJECT_NAME})
    set(SIGHT_MODULE_RC_PREFIX "${CMAKE_INSTALL_DATADIR}/${FW_INSTALL_PATH_SUFFIX}")
    if(WIN32)
        set(SIGHT_MODULE_LIB_PREFIX "${CMAKE_INSTALL_BINDIR}")
    else()
        set(SIGHT_MODULE_LIB_PREFIX "${CMAKE_INSTALL_LIBDIR}")
    endif()

    file(TO_CMAKE_PATH "${sight_DIR}/../../../bin" Sight_BINARY_DIR)
    file(TO_CMAKE_PATH "${sight_DIR}/../../../lib/x86_64-linux-gnu" Sight_LIBRARY_DIR)
    file(TO_CMAKE_PATH "${sight_DIR}/../../../share/sight" Sight_MODULES_DIR)
    file(TO_CMAKE_PATH "${sight_DIR}/../../../" Sight_DEFINE_DIR)

    setExternalLibrariesDir()

    find_package(CppUnit REQUIRED)

    foreach(sight_comp ${SIGHT_SUPPORTED_COMPONENTS})
        if(NOT ${sight_comp} MATCHES "_obj")
            if(EXISTS "${CMAKE_CURRENT_LIST_DIR}/sight_${sight_comp}_Dependencies.cmake")
                include("${CMAKE_CURRENT_LIST_DIR}/sight_${sight_comp}_Dependencies.cmake")
            endif()
            include("${CMAKE_CURRENT_LIST_DIR}/sight_${sight_comp}_Targets.cmake")
        endif()
        list(APPEND SIGHT_COMPONENTS sight::${sight_comp})
    endforeach()
endif()

foreach(sight_comp ${sight_FIND_COMPONENTS})
    if(NOT ${sight_comp}_FOUND)
        message(STATUS "Looking for sight component: ${sight_comp}")
        if (NOT sight_comp IN_LIST SIGHT_SUPPORTED_COMPONENTS)
            set(sight_FOUND False)
            set(sight_NOT_FOUND_MESSAGE "Unsupported component: ${sight_comp}")
        endif()

        set_property(TARGET sight::${sight_comp} PROPERTY INTERFACE_COMPILE_DEFINITIONS SIGHT_DIR="${Sight_DEFINE_DIR}")
        set(${sight_comp}_FOUND 1)
    endif()
endforeach()
